// Desktop Rox - A game by Stephen Stair (sgstair@akkit.org) for LudumDare 11 (48h game competition)
// Bitmap.cpp - Bitmap class to encapsulate Windows GDI bitmap drawing, and provide useful tools for interacting with bitmaps

#include "Bitmap.h"
#include <cstring>
using namespace std;

Color::Color()
{
	c=0;
}
Color::Color(unsigned long c_)
{
	c=c_;
}
Color::Color(int r, int g, int b)
{
	if(r<0) r=0; if(g<0) g=0; if(b<0) b=0;
	if(r>255) r=255; if(g>255) g=255; if(b>255) b=255;

	c = (r<<16) | (g<<8) | (b) | 0xFF000000;
}

Color::Color(int a, int r, int g, int b)
{
	if(a<0) a=0; if(r<0) r=0; if(g<0) g=0; if(b<0) b=0;
	if(a>255) a=255; if(r>255) r=255; if(g>255) g=255; if(b>255) b=255;
	c = (a<<24) | (r<<16) | (g<<8) | (b);
}


void Color::apply(Color topcolor)
{
	int ra,ga,ba,aa,rb,gb,bb,ab;
	aa=(c>>24)&255; ra = (c>>16)&255; ga=(c>>8)&255; ba=c&255;
	ab=(topcolor.c>>24)&255; rb = (topcolor.c>>16)&255; gb=(topcolor.c>>8)&255; bb=topcolor.c&255;
	int ta = aa+ab;
	if(ta==0) return;
	ra = ((ra*aa) + (rb*ab))/ta;
	ga = ((ga*aa) + (gb*ab))/ta;
	ba = ((ba*aa) + (bb*ab))/ta;
	aa = ba + ((255-ba)*aa+127)/255;
	c = (aa<<24) | (ra<<16) | (ga<<8) | (ba);
}
void Color::blend(Color topcolor)
{
	int ra,ga,ba,aa,rb,gb,bb,ab;
	aa=(c>>24)&255; ra = (c>>16)&255; ga=(c>>8)&255; ba=c&255;
	ab=(topcolor.c>>24)&255; rb = (topcolor.c>>16)&255; gb=(topcolor.c>>8)&255; bb=topcolor.c&255;
	if(ab==0) return;
	aa = 255-ab;
	ra = ((ra*aa) + (rb*ab))/255;
	ga = ((ga*aa) + (gb*ab))/255;
	ba = ((ba*aa) + (bb*ab))/255;
	c = 0xFF000000 | (ra<<16) | (ga<<8) | (ba);
}


Bitmap::Bitmap(int w_, int h_)
{
	MakeBmp(w_,h_);
}
Bitmap::~Bitmap(void)
{
	DestroyBmp(hDC,hBmp);
	hDC=0; hBmp=0;
	w=0;h=0;
	pBits=0;
}

void Bitmap::resize(int neww, int newh)
{
	if(w==neww && h==newh) return;
	DestroyBmp(hDC, hBmp);
	MakeBmp(neww,newh);
}

HDC Bitmap::DC()
{
	return hDC;
}
unsigned long * Bitmap::bits()
{
	return pBits;
}
int Bitmap::wid()
{
	return w;
}
int Bitmap::hei()
{
	return h;
}

void Bitmap::cls(Color c)
{
	int i;
	if(c.c==0)
	{
		memset(pBits,0,w*h*4);
	}
	else
	{
		for(i=0;i<w*h;i++)
		{
			pBits[i] = c.c;
		}
	}
}

void Bitmap::pset(int x, int y, Color c)
{
	if(x<0 || y<0 || x>=w || y>=h) return;
	pBits[x+y*w] = c.c;
}



#define line_abs(n) ((n)<0?-(n):(n))
void Bitmap::line( int x1, int y1, int x2, int y2, Color c )
{
	int dx,dy;
	dx = x2-x1;
	dy = y2-y1;
	if(line_abs(dx)>=line_abs(dy))
	{ // stepping in x direction
		if(dx==0) { pset(x1,y1,c); return; } // single point
		if(x2<x1)
		{
			x1^=x2; x2^=x1; x1^=x2;
			dx=-dx; dy=-dy; y1=y2;	
		}
		y1 = y1<<16;
		dy = (dy<<16)/dx;
		y1 += (dy>>1)+0x8000;
		if(x1<0) {y1-=dy*x1; x1=0;}
		if(x2>=w) x2=w-1;
		for(;x1<=x2;x1++)
		{
			if(y1>=0 && (y1>>16)<h) pBits[x1+(y1>>16)*w] = c.c;
			y1+=dy;
		}
	}
	else
	{	// stepping in y direction
		if(y2<y1)
		{
			y1^=y2; y2^=y1; y1^=y2;
			dx=-dx; dy=-dy; x1=x2;	
		}
		x1 = x1<<16;
		dx = (dx<<16)/dy;
		x1 += (dx>>1)+0x8000;
		if(y1<0) {x1-=dx*y1; y1=0;}
		if(y2>=h) y2=h-1;
		for(;y1<=y2;y1++)
		{
			if(x1>=0 && (x1>>16)<h) pBits[(x1>>16)+y1*w] = c.c;
			x1+=dx;
		}
	}
}




// render with alphablending!
void Bitmap::render( int x, int y, Bitmap & bmp )
{
	int rx,ry,rw,rh;
	int ox,oy;
	int pl, pr;
	unsigned long px;

	rx=0; ry=0;	rw = bmp.w;	rh = bmp.h;
	if(x<0) {		rx-=x;		rw+=x;		x=0;	}
	if(y<0) {		ry-=y;		rh+=y;		y=0;	}
	if(x+rw>=w)	{	rw = w-x;	}
	if(y+rh>=h)	{	rh = h-y;	}

	for(oy=0;oy<rh;oy++)
	{
		pl = (y+oy)*w + x;
		pr = (ry+oy)*bmp.w + rx;
		for(ox=0;ox<rw;ox++)
		{
			px = bmp.pBits[pr++];
			((Color *)pBits)[pl++].blend(px);
		}
	}
}






void Bitmap::MakeLayeredAlpha()
{
	int i;
	unsigned long px;
	int r,g,b,a;
	for(i=0;i<w*h;i++)
	{
		px = pBits[i];
		a = (px>>24)&255;	r = (px>>16)&255;	g = (px>>8)&255;	b = (px)&255;		
		r=(r*a)/255;
		g=(g*a)/255;
		b=(b*a)/255;
		pBits[i] = (a<<24) | (r<<16) | (g<<8) | (b);
	}
}






void Bitmap::MakeBmp(int w_, int h_)
{
	BITMAPINFOHEADER bih;
	
	w = w_;
	h = h_;

	bih.biBitCount=32;
	bih.biClrImportant=0;
	bih.biClrUsed=0;
	bih.biCompression=BI_RGB;
	bih.biHeight=-h;
	bih.biPlanes=1;
	bih.biSize=sizeof(bih);
	bih.biSizeImage=0;
	bih.biWidth=w;
	bih.biXPelsPerMeter=0;
	bih.biYPelsPerMeter=0;

	HDC hTempDC;

	hTempDC = GetDC( GetDesktopWindow() );
	
	pBits=0;
	hBmp = CreateDIBSection(hTempDC, (BITMAPINFO *) &bih, 0, (void **)&pBits, 0, 0 );

	hDC = CreateCompatibleDC(hTempDC);
	ReleaseDC( GetDesktopWindow(), hTempDC );
	
	SelectObject(hDC, hBmp);
}

void Bitmap::DestroyBmp(HDC hDC_, HBITMAP hBmp_)
{
	DeleteDC(hDC_);
	DeleteObject(hBmp_);
}
