// Desktop Rox - A game by Stephen Stair (sgstair@akkit.org) for LudumDare 11 (48h game competition)
// main.cpp - Main game setup / game loop. Not much else.

#include <windows.h>

#include <cmath>

#include <algorithm>
#include <vector>
using namespace std;

#include "Bitmap.h"


#define MY_PI 3.1415926535897932384626433832795

HINSTANCE g_hInst;

const double desired_fps = 30.0;
const double frame_time = 1.0 / desired_fps;

double timer_resolution;
double game_time;
__int64 start_time;
__int64 last_frame_time;
__int64 cur_game_time;



double RandomNumber()
{ // create random number between 0 and 1.
	double n = 0;
	__int64 z = (__int64)rand() ^ ((__int64)rand()<<8) ^ ((__int64)rand()<<16) ^ ((__int64)rand()<<24) ^ ((__int64)rand()<<32) ^ ((__int64)rand()<<40) ^ ((__int64)rand()<<48) ^ ((__int64)rand()<<56);
	
	n = (z&0x7FFFFFFFFFFFFFFFll) / (double) 0x7FFFFFFFFFFFFFFFll;
	return n;
}

double RandomApproxGaussian(int iterations = 16)
{
	double n = 0.0;
	for(int i=0;i<iterations;i++)
		n += RandomNumber();
	return n / iterations;
}


#include "SpriteManager.h"

SpriteManager * g_SM = 0;

#include "BitmapSprite.h"

#include "Rasterizer.h"


Rasterizer * g_Ras = 0;

//////////////////////////////////////////////////////////////////////////
// Handle sprites!

vector<BitmapSprite *> g_sprites;

enum SpriteType {
	ST_BITMAP
};


void RemoveSprite( BitmapSprite * bs )
{
	vector<BitmapSprite *>::iterator i;
	i = find(g_sprites.begin(),g_sprites.end(),bs);
	if(i!=g_sprites.end())
	{
		g_sprites.erase(i);
		if(g_SM) g_SM->RemoveSprite(bs);
	}
}

void AddSprite( BitmapSprite * bs )
{
	RemoveSprite(bs); // precaution
	g_sprites.push_back(bs);
	if(g_SM) g_SM->AddSprite(bs);
}

BitmapSprite * AddSprite(SpriteType st, int x=0, int y=0)
{
	BitmapSprite * bs;
	switch(st)
	{
	case ST_BITMAP:
	default:
		bs = new BitmapSprite();
	}
	bs->bx = x;
	bs->by = y;
	bs->alpha=255;
	AddSprite(bs);
	return bs;
}

void DestroySprite( BitmapSprite * bs )
{
	RemoveSprite(bs);
	delete bs;
}

void DestroyAllSprites()
{
	for(size_t i = 0; i<g_sprites.size();i++) 
	{
		if(g_SM) g_SM->RemoveSprite(g_sprites[i]);
		delete g_sprites[i];
	}
	g_sprites.clear();
}




//////////////////////////////////////////////////////////////////////////
// Handle SpriteManager instance

#include "WindowSpriteManager.h"
#include "DesktopSpriteManager.h"

enum SpriteManagerType {
	SMT_WINDOW	=0,
	SMT_DESKTOP	=1
};


void SetSpriteManager(SpriteManagerType smt)
{
	// remove sprites from old manager
	size_t i;
	if(g_SM)
	{
		for(i=0;i<g_sprites.size();i++)
		{
			g_SM->RemoveSprite(g_sprites[i]);
		}
	}
	delete g_SM;
	g_SM=0;
	switch(smt)
	{
	case SMT_WINDOW:
		g_SM = new WindowSpriteManager(800,600);
		break;
	case SMT_DESKTOP:
	default:
		g_SM = new DesktopSpriteManager();
		break;
	}
	// add sprites to new manager!
	for(i=0;i<g_sprites.size();i++)
	{
		g_SM->AddSprite(g_sprites[i]);
	}
}


//////////////////////////////////////////////////////////////////////////
// Handle Rasterizer instance!


#include "OriginalRasterizer.h"

enum RasterizerType {
	RT_ORIGINAL
};

void SetRasterizer(RasterizerType rt)
{
	delete g_Ras;
	g_Ras=0;
	switch(rt)
	{
	case RT_ORIGINAL:
	default:
		g_Ras = new OriginalRasterizer();
	}
}


//////////////////////////////////////////////////////////////////////////
// Helper-BlockFont!



char blockfontdata[] =
{ // 3x5 "pixels" per character
	// '0'
	0,1,0,
	1,0,1,
	1,0,1,
	1,0,1,
	0,1,0,
	// '1'
	0,1,0,
	1,1,0,
	0,1,0,
	0,1,0,
	1,1,1,
	// '2'
	0,1,0,
	1,0,1,
	0,0,1,
	0,1,0,
	1,1,1,
	// '3'
	1,1,0,
	0,0,1,
	1,1,0,
	0,0,1,
	1,1,0,
	// '4'
	1,0,1,
	1,0,1,
	1,1,1,
	0,0,1,
	0,0,1,
	// '5'
	1,1,1,
	1,0,0,
	1,1,0,
	0,0,1,
	1,1,0,
	// '6'
	0,1,1,
	1,0,0,
	1,1,0,
	1,0,1,
	1,1,1,
	// '7'
	1,1,0,
	0,0,1,
	0,1,0,
	0,1,0,
	0,1,0,
	// '8'
	0,1,0,
	1,0,1,
	0,1,0,
	1,0,1,
	0,1,0,
	// '9'
	0,1,0,
	1,0,1,
	0,1,1,
	0,0,1,
	0,0,1,
	// 'A'
	0,1,0,
	1,0,1,
	1,1,1,
	1,0,1,
	1,0,1,
	// 'B'
	1,1,0,
	1,0,1,
	1,1,1,
	1,0,1,
	1,1,0,
	// 'C'
	0,1,1,
	1,0,0,
	1,0,0,
	1,0,0,
	0,1,1,
	// 'D'
	1,1,0,
	1,0,1,
	1,0,1,
	1,0,1,
	1,1,0,
	// 'E'
	1,1,1,
	1,0,0,
	1,1,0,
	1,0,0,
	1,1,1,
	// 'F'
	1,1,1,
	1,0,0,
	1,1,0,
	1,0,0,
	1,0,0,
	// 'G'
	0,1,1,
	1,0,0,
	1,0,1,
	1,0,1,
	0,1,0,
	// 'H'
	1,0,1,
	1,0,1,
	1,1,1,
	1,0,1,
	1,0,1,
	// 'I'
	1,1,1,
	0,1,0,
	0,1,0,
	0,1,0,
	1,1,1,
	// 'J'
	1,1,1,
	0,0,1,
	0,0,1,
	1,0,1,
	0,1,0,
	// 'K'
	1,0,1,
	1,0,1,
	1,1,0,
	1,0,1,
	1,0,1,
	// 'L'
	1,0,0,
	1,0,0,
	1,0,0,
	1,0,0,
	1,1,1,
	// 'M'
	1,0,1,
	1,1,1,
	1,0,1,
	1,0,1,
	1,0,1,
	// 'N'
	1,0,1,
	1,0,1,
	1,1,1,
	1,1,1,
	1,0,1,
	// 'O'
	1,1,1,
	1,0,1,
	1,0,1,
	1,0,1,
	1,1,1,
	// 'P'
	1,1,0,
	1,0,1,
	1,1,0,
	1,0,0,
	1,0,0,
	// 'Q'
	1,1,1,
	1,0,1,
	1,0,1,
	1,1,0,
	1,0,1,
	// 'R'
	1,1,0,
	1,0,1,
	1,1,0,
	1,0,1,
	1,0,1,
	// 'S'
	0,1,1,
	1,0,0,
	0,1,1,
	0,0,1,
	1,1,1,
	// 'T'
	1,1,1,
	0,1,0,
	0,1,0,
	0,1,0,
	0,1,0,
	// 'U'
	1,0,1,
	1,0,1,
	1,0,1,
	1,0,1,
	1,1,1,
	// 'V'
	1,0,1,
	1,0,1,
	1,0,1,
	1,0,1,
	0,1,0,
	// 'W'
	1,0,1,
	1,0,1,
	1,0,1,
	1,1,1,
	1,0,1,
	// 'X'
	1,0,1,
	1,0,1,
	0,1,0,
	1,0,1,
	1,0,1,
	// 'Y'
	1,0,1,
	1,0,1,
	0,1,0,
	0,1,0,
	0,1,0,
	// 'Z'
	1,1,1,
	0,0,1,
	0,1,0,
	1,0,0,
	1,1,1,
	// ':'
	0,0,0,
	1,0,0,
	0,0,0,
	1,0,0,
	0,0,0,
	// '!'
	1,0,0,
	1,0,0,
	1,0,0,
	0,0,0,
	1,0,0,
	// '.'
	0,0,0,
	0,0,0,
	0,0,0,
	0,0,0,
	1,0,0,
	// "%"
	1,0,1,
	0,0,1,
	0,1,0,
	1,0,0,
	1,0,1,

};
 // char blockfontdata[] (3*5 values per character)
char * blockfontchars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ:!.%";
int mapping[256];


void SetupBlockFont()
{
	int i;
	for(i=0;i<255;i++) mapping[i]=-1;
	for(i=0;blockfontchars[i];i++)
	{
		mapping[blockfontchars[i]] = i;
	}
}


void RenderString( vector<RenderRectangle> & boxlist, double xstart, double ystart, double blocksize, double charwidth, char * s)
{
	int c,x,y,n;
	double blockspacing = charwidth/3.5;
	RenderRectangle rr;
	rr.w = blocksize;
	rr.h = blocksize;
	for(c=0;s[c];c++)
	{
		if( (n=mapping[s[c]]) != -1 )
		{
			n=n*15;
			for(y=0;y<5;y++)
			{
				for(x=0;x<3;x++)
				{
					if( blockfontdata[n+x+y*3] )
					{
						rr.x = xstart + (x+0.5)*blockspacing;
						rr.y = ystart + (y+0.5)*blockspacing;
						boxlist.push_back(rr);
					}
				}
			}
		}
		xstart+=charwidth;
	}
}












//////////////////////////////////////////////////////////////////////////
// Less support code, more game code after this point

const int game_start_num_asteroids = 6;
const int game_asteroid_spawn_delay = 10;

const int game_difficultyrank = 1000;
const int game_autofire_frames = 5;

double game_baseasteroidspeed = 3.0;
double game_asteroidspeed = 3.0;
double game_playerspeed = 6.0;
double game_bulletspeed = 9.0;
double game_asteroidradius = 150.0;
double game_asteroidvariance = 0.2;
double game_asteroidstartdistance = 200.0;
double game_killdistance = 250.0;
double game_playersize = 75.0;
double game_bulletsize = 40.0;

int game_cur_num_asteroids = 8;
int game_asteroid_spawn_delay_timer = 0;

int game_base_asteroid_edges = 8;
int game_asteroid_edges_variance = 8;

int game_asteroid_fragments = 4;

double player_x, player_y, player_rotation;
double player_vx, player_vy;


int game_score_little_asteroid = 5;
int game_score_big_asteroid = 2;

int game_autofire_count = 0;

int game_difficultypoints;
double game_difficulty;

int game_score;

void AddScore( int points )
{
	game_score+=points;
	game_difficultypoints += points;
}
void ShotFired()
{
	game_difficultypoints += 1;
	game_score--;
	if(game_score<0) game_score=0;
}
void AsteroidSpawned()
{
	game_difficultypoints += 2;
}

enum GameState
{
	GS_MENU,
	GS_INGAME,
	GS_PAUSING,
	GS_PAUSED,
	GS_UNPAUSING,
	GS_LOSING
};

GameState g_gs = GS_INGAME;



//////////////////////////////////////////////////////////////////////////
// Bullet management!

class Bullet {
public:
	BitmapSprite * mysprite;
	double x,y,heading, speed, radius;
	double bullettime;
};

vector<Bullet> bulletlist;

void KillBullet( int index )
{
	DestroySprite(bulletlist[index].mysprite);

	bulletlist.erase( bulletlist.begin() + index );
}
void DestroyBullets()
{
	while(!bulletlist.empty())
	{
		KillBullet( (int)(bulletlist.size()-1));
	}
}

void SpawnBullet( double x, double y, double heading, double lead_distance = 0.0)
{
	Bullet b;
	b.mysprite = AddSprite(ST_BITMAP);
	b.mysprite->bmp.resize((int)game_bulletsize*3,(int)game_bulletsize*3);
	b.x = x;
	b.y = y;
	b.heading = heading;
	b.radius = game_bulletsize;
	b.speed = game_bulletspeed;
	b.x += cos(heading)*lead_distance;
	b.y += sin(heading)*lead_distance;
	b.bullettime=RandomNumber()*6;

	bulletlist.push_back(b);
}

void UpdateBullets()
{
	size_t i;
	int w,h;
	w = g_SM->GetFieldWidth();
	h = g_SM->GetFieldHeight();

	if(g_gs==GS_INGAME)
	{
		// cull out-of-bounds bullets
		for(i=0;i<bulletlist.size();i++)
		{
			if(bulletlist[i].x<-game_bulletsize*2 || bulletlist[i].y<-game_bulletsize*2 ||
				bulletlist[i].x>w+game_bulletsize*2 || bulletlist[i].y>h+game_bulletsize*2 )
			{
				KillBullet((int)i);
				i--;
			}
		}

		// move bullets
		for(i=0;i<bulletlist.size();i++)
		{
			bulletlist[i].x += cos(bulletlist[i].heading)*bulletlist[i].speed;
			bulletlist[i].y += sin(bulletlist[i].heading)*bulletlist[i].speed;
		}

		RenderRectangle rr;
		rr.x=0;
		rr.y=0;
		rr.w=game_bulletsize;
		rr.h=game_bulletsize/3;

		// render bullets!
		for(i=0;i<bulletlist.size();i++)
		{
			bulletlist[i].bullettime+=0.2;
			double wiggle,bt;
			bt = bulletlist[i].bullettime;
			wiggle= 0.3 * sin(bt*8) * ( ((1-cos(bt))/2)*((1-cos(bt))/2) );

			bulletlist[i].mysprite->bx = (int)floor(bulletlist[i].x) - bulletlist[i].mysprite->bmp.wid()/2;
			bulletlist[i].mysprite->by = (int)floor(bulletlist[i].y) - bulletlist[i].mysprite->bmp.hei()/2;
			g_Ras->Bind( &bulletlist[i].mysprite->bmp, bulletlist[i].mysprite->bx - bulletlist[i].x, bulletlist[i].mysprite->by - bulletlist[i].y );
			g_Ras->DrawRectangles(1,&rr,bulletlist[i].heading+wiggle);
			g_Ras->FlushChanges();
		}
	}


}

//////////////////////////////////////////////////////////////////////////
// Asteroid Management!

enum AsteroidState
{
	AS_NORMAL,
	AS_DYING
};

class Asteroid {
public:
	BitmapSprite * mysprite;
	double radius;
	double mass;
	double rotation;
	double vrot;
	double vx,vy;
	double x,y;
	AsteroidState state;
	vector<RenderPoint> outline;
};

vector<Asteroid> asteroidlist;


void KillAsteroid(int index)
{
	DestroySprite(asteroidlist[index].mysprite);

	asteroidlist.erase( asteroidlist.begin() + index );
}

void DestroyAsteroids()
{
	while(!asteroidlist.empty())
	{
		KillAsteroid((int)(asteroidlist.size()-1));
	}
}

void AddRandomAsteroid(double radius)
{
	Asteroid a;
	double var;
	int n;
	int w,h;
	n=rand()%3;
	a.mass = radius*radius;
	a.mysprite = AddSprite(ST_BITMAP);
	a.radius = radius;
	var = radius * game_asteroidvariance;

	a.mysprite->bmp.resize( (int)(radius+var+5)*2, (int)(radius+var+5)*2 );

	a.rotation=0;
	a.vrot = RandomNumber()*0.1-0.05;
	a.state=AS_NORMAL;

	w = g_SM->GetFieldWidth();
	h = g_SM->GetFieldHeight();
	if((n&2)==0)
	{ // along top/bottom
		a.x = -game_asteroidstartdistance + RandomNumber()*(w+game_asteroidstartdistance*2);
		if(n&1) a.y = -game_asteroidstartdistance; else a.y=h+game_asteroidstartdistance;
	}
	else
	{ // along left/right
		a.y = -game_asteroidstartdistance + RandomNumber()*(h+game_asteroidstartdistance*2);
		if(n&1) a.x = -game_asteroidstartdistance; else a.x=w+game_asteroidstartdistance;
	}

	// pick a random point in the screen to go towards
	double vx,vy, v;
	vx = RandomNumber()*w - a.x;
	vy = RandomNumber()*h - a.y;
	v = sqrt(vx*vx+vy*vy) / (RandomApproxGaussian()*0.5 + 0.75);
	a.vx = vx/v;
	a.vy = vy/v;

	n = game_base_asteroid_edges + rand()%game_asteroid_edges_variance;
	vector<double> locations;
	locations.reserve(n);
	locations.push_back(RandomApproxGaussian());
	int i;
	int divide_span;
	double max_span, prev_loc, base_loc;
	while((int)locations.size()<n)
	{ // find largest span and subdivide it
		max_span = 1.0-locations.back();
		base_loc = locations.back();
		prev_loc=0.0;
		divide_span=(int)locations.size();
		for(i=0;i<(int)locations.size();i++)
		{
			if(locations[i]-prev_loc > max_span)
			{
				base_loc = prev_loc;
				max_span = locations[i]-prev_loc;
				divide_span = i;
			}
			prev_loc = locations[i];
		}
		if(max_span==0) break; // somehow we're not going to get any further.
		locations.insert( locations.begin()+divide_span, RandomApproxGaussian()*max_span+base_loc);
	}
	// now construct points!
	RenderPoint rp;
	double rad;
	a.outline.reserve(locations.size());
	for(i=0;i<(int)locations.size();i++)
	{
		rad = radius + var*RandomNumber()-var/2;
		rp.x = cos( locations[i] * 2 * MY_PI ) * rad;
		rp.y = sin( locations[i] * 2 * MY_PI ) * rad;
		a.outline.push_back(rp);
	}
	
	asteroidlist.push_back(a);
}

void UpdateAsteroids()
{
	size_t i,j,k;
	int w,h;
	w = g_SM->GetFieldWidth();
	h = g_SM->GetFieldHeight();

	if(g_gs==GS_INGAME)
	{
		if(game_asteroid_spawn_delay_timer>0) game_asteroid_spawn_delay_timer--;
		if((int)asteroidlist.size()<game_cur_num_asteroids && game_asteroid_spawn_delay_timer==0)
		{
			AsteroidSpawned();
			AddRandomAsteroid(game_asteroidradius);
			game_asteroid_spawn_delay_timer = game_asteroid_spawn_delay;
		}

		// cull out-of-bounds asteroids
		for(i=0;i<asteroidlist.size();i++)
		{
			if(asteroidlist[i].x<-game_killdistance || asteroidlist[i].y<-game_killdistance ||
				asteroidlist[i].x>w+game_killdistance || asteroidlist[i].y>h+game_killdistance )
			{
				KillAsteroid((int)i);
				i--;
			}
		}

		// check bullet collision
		double vx,vy,r,nx, ny, v;
		double dx,dy;
		for(i=0;i<asteroidlist.size();i++)
		{
			for(j=0;j<bulletlist.size();j++)
			{
				dx = bulletlist[j].x-asteroidlist[i].x;
				dy = bulletlist[j].y-asteroidlist[i].y;
				r = sqrt(dx*dx+dy*dy);
				if(r<(bulletlist[j].radius+asteroidlist[i].radius))
				{ // Boom!
					KillBullet(j);
					j--;

					if(asteroidlist[i].radius==game_asteroidradius)
					{
						AddScore(game_score_big_asteroid);
						vx = asteroidlist[i].vx;
						vy = asteroidlist[i].vy;
						v = sqrt(vx*vx+vy*vy);
						for(k=0;k<game_asteroid_fragments;k++)
						{
							dx = cos( ((double)k/game_asteroid_fragments)*MY_PI*2 + game_time);
							dy = sin( ((double)k/game_asteroid_fragments)*MY_PI*2 + game_time);
							AddRandomAsteroid( asteroidlist[i].radius/2);
							asteroidlist.back().x = asteroidlist[i].x + dx*asteroidlist[i].radius/2;
							asteroidlist.back().y = asteroidlist[i].y + dy*asteroidlist[i].radius/2;
							asteroidlist.back().vx = dx*v*1.8;
							asteroidlist.back().vy = dy*v*1.8;
							asteroidlist.back().vrot = asteroidlist[i].vrot*1.8;
						}
					}
					else
					{
						AddScore(game_score_big_asteroid);
					}
					KillAsteroid(i);
					i--;
					break;
				}
			}
		}


		// move asteroids
		for(i=0;i<asteroidlist.size();i++)
		{
			vx=asteroidlist[i].vx;
			vy=asteroidlist[i].vy;
			v = sqrt(vx*vx+vy*vy);
			if(v==0) continue;
			nx=vy/v;
			ny=-vx/v;
			if(v<0.5) v=v*1.05; // speed hack!
			r = -asteroidlist[i].vrot/20;
			vx += nx*r;
			vy += ny*r;
			r = sqrt(vx*vx+vy*vy);
			vx = vx * v / r;
			vy = vy * v / r;
			asteroidlist[i].vx = vx;
			asteroidlist[i].vy = vy;
			asteroidlist[i].x += vx*game_asteroidspeed;
			asteroidlist[i].y += vy*game_asteroidspeed;

			asteroidlist[i].rotation += asteroidlist[i].vrot;
		}

		// check asteroid collision
		double vi,vj, fi, fj, mti, mtj, rti, rtj;
		for(i=0;i<asteroidlist.size();i++)
		{
			for(j=i+1;j<asteroidlist.size();j++)
			{
				dx = asteroidlist[j].x - asteroidlist[i].x;
				dy = asteroidlist[j].y - asteroidlist[i].y;
				r = sqrt(dx*dx+dy*dy);
				if(r< asteroidlist[j].radius + asteroidlist[i].radius)
				{
					vi = sqrt(asteroidlist[i].vx*asteroidlist[i].vx+asteroidlist[i].vy*asteroidlist[i].vy);
					vj = sqrt(asteroidlist[j].vx*asteroidlist[j].vx+asteroidlist[j].vy*asteroidlist[j].vy);
					fi = (dx*asteroidlist[i].vx + dy*asteroidlist[i].vy)/(vi*r+0.1); // approximate percent force; amount of inertia to transfer
					fj = (-dx*asteroidlist[j].vx - dy*asteroidlist[j].vy)/(vj*r+0.1);
					fi = fi * 0.98; // enforce maximum velocity transfer, can't have things stopping dead in the water.
					fj = fj * 0.98; 
					if(fi<0) fi=0;
					if(fj<0) fj=0;
					mti = vi * fi * asteroidlist[i].mass; // sound volume can be mti+mtj :P
					mtj = vj * fj * asteroidlist[j].mass;
					rti = fi * asteroidlist[i].mass * asteroidlist[i].vrot;
					rtj = fj * asteroidlist[j].mass * asteroidlist[j].vrot;

					// apply velocity changes! Note that momentum is roughly conserved, so things shouldn't get too out of control
					//  (note to self: probably famous last words)
					asteroidlist[i].vx = asteroidlist[i].vx * (1-fi) - (dx/r)*(mtj/asteroidlist[i].mass);
					asteroidlist[i].vy = asteroidlist[i].vy * (1-fi) - (dy/r)*(mtj/asteroidlist[i].mass);
					asteroidlist[j].vx = asteroidlist[j].vx * (1-fj) + (dx/r)*(mti/asteroidlist[j].mass);
					asteroidlist[j].vy = asteroidlist[j].vy * (1-fj) + (dy/r)*(mti/asteroidlist[j].mass);

					asteroidlist[i].vrot = asteroidlist[i].vrot * (1-fi) - rtj/asteroidlist[i].mass;
					asteroidlist[j].vrot = asteroidlist[j].vrot * (1-fj) - rti/asteroidlist[j].mass;
				}
				
			}
		}

		// render asteroids!
		for(i=0;i<asteroidlist.size();i++)
		{
			asteroidlist[i].mysprite->bx = (int)floor(asteroidlist[i].x) - asteroidlist[i].mysprite->bmp.wid()/2;
			asteroidlist[i].mysprite->by = (int)floor(asteroidlist[i].y) - asteroidlist[i].mysprite->bmp.hei()/2;
			g_Ras->Bind( &asteroidlist[i].mysprite->bmp, asteroidlist[i].mysprite->bx - asteroidlist[i].x, asteroidlist[i].mysprite->by - asteroidlist[i].y );
			g_Ras->DrawClosedFigure((int)asteroidlist[i].outline.size(), &asteroidlist[i].outline[0], asteroidlist[i].rotation );
			g_Ras->FlushChanges();
		}
	}

}


//////////////////////////////////////////////////////////////////////////
// Game management!





double line_thickness = 12.0;
double shadow_thickness = 6.0;

vector<BoundaryPoint> screen_boundary;

int menuwidth, menuheight;
int scorewidth,scoreheight;

int menux,menuy,scorex,scorey;

void SetupMenu()
{
	RECT rc = g_SM->GetMainWindow();
	int height = rc.bottom-rc.top;
	int width = rc.right-rc.left;

	menuwidth=width/3;
	menuheight=height/3;
	if(menuwidth<600) menuwidth=600;
	if(menuheight<400) menuheight=400;
	if(menuheight<menuwidth*2/3) menuheight=menuwidth*2/3;

	menux = (width-menuwidth)/2 + rc.left;
	menuy = (height-menuheight)/2 + rc.top;

	scorex = rc.left;
	scorey = rc.top;
	scorewidth = width/4;
	scoreheight = height/8;;
	if(scorewidth<400) scorewidth=400;
	if(scoreheight<200) scoreheight=200;

}

void SetupGame()
{
	DestroyAsteroids();
	DestroyBullets();

	game_score = 0;
	screen_boundary = g_SM->GetFieldBoundary();
	RECT rc = g_SM->GetMainWindow();
	int height = rc.bottom-rc.top;

	// set game sizes / speeds based on main window height!
	game_asteroidradius = (double)height/8.0;
	game_asteroidstartdistance= game_asteroidradius+50;
	game_killdistance = game_asteroidradius+100;

	line_thickness=(double)height/100.0;
	shadow_thickness=line_thickness/2;

	game_cur_num_asteroids = game_start_num_asteroids;

	game_baseasteroidspeed = game_asteroidspeed = game_asteroidradius / 50.0;
	game_playerspeed = game_asteroidspeed*2;
	game_bulletspeed = game_playerspeed*7;

	game_playersize = game_asteroidradius/2;
	game_bulletsize = game_playersize/2;

	player_x = (rc.left+rc.right)/2;
	player_y = (rc.bottom+rc.top)/2;
	player_rotation=0;
	player_vx = player_vy = 0;

	game_difficultypoints=0;
	game_difficulty=1;
}


RenderRectangle btnstart,btnexit;



BitmapSprite * playerSprite = 0;
BitmapSprite * menuSprite = 0;
BitmapSprite * scoreSprite = 0;

double v_killing_object = 0;

void SetMenuMode()
{
	DestroyAsteroids();
	DestroyBullets();
	playerSprite->alpha=0;
	scoreSprite->alpha=0;
	menuSprite->alpha=255;
	g_gs = GS_MENU;
}

void SetIngameMode()
{
	SetupGame();
	playerSprite->alpha=255;
	scoreSprite->alpha=255;
	menuSprite->alpha=0;
	g_gs = GS_INGAME;
}

void SetLosingMode()
{
	DestroyAsteroids();
	DestroyBullets();
	playerSprite->alpha=0;
	scoreSprite->alpha=0;
	menuSprite->alpha=255;
	g_gs = GS_LOSING;

}

void ClickHandler(int x, int y, BitmapSprite * src)
{
	if(g_gs==GS_MENU)
	{
		double dx,dy;
		dx = x - btnstart.x;
		if(dx<0) dx=-dx;
		dy = y - btnstart.y;
		if(dy<0) dy=-dy;

		if(dx<btnstart.w/2 && dy<btnstart.h/2) {
			SetIngameMode();
			return;
		}

		dx = x - btnexit.x;
		if(dx<0) dx=-dx;
		dy = y - btnexit.y;
		if(dy<0) dy=-dy;

		if(dx<btnexit.w/2 && dy<btnexit.h/2) {
			exit(0);
			return;
		}
	}
	if(g_gs==GS_LOSING)
	{
		SetMenuMode();
	}

}


bool fire_down=false, fire_was_down=false;
// much to do here!
void UpdateGameLogic()
{
	// preliminary game logic
	switch(g_gs)
	{
	case GS_INGAME:
		if(GetFocus()==NULL)
		{ // need to pause?
			g_gs = GS_PAUSED;
		}
		break;
	case GS_PAUSED:
		if(GetFocus()!=NULL)
		{ // need to pause?
			g_gs = GS_INGAME;
		}
		break;

	}

	// set difficulty
	game_difficulty = 1 + (double)game_difficultypoints/game_difficultyrank;
	game_cur_num_asteroids = (int)(game_start_num_asteroids * game_difficulty);
	game_asteroidspeed = game_baseasteroidspeed * game_difficulty;

	// update asteroids
	UpdateAsteroids();

	// update bullets
	UpdateBullets();

	// update player
	{
		if(g_gs==GS_INGAME)
		{
			// check for death!
			size_t i,j;
			double r,dx,dy,nx,ny,d,n;
			r = game_playersize /3;
			for(i=0;i<asteroidlist.size();i++)
			{
				dx = player_x - asteroidlist[i].x;
				dy = player_y - asteroidlist[i].y;
				d = sqrt(dx*dx+dy*dy);
				if(d<asteroidlist[i].radius + r )
				{ // Certain DOOM!
					// preserve velocity of asteroid for later ridicule
					v_killing_object = game_asteroidspeed * sqrt(asteroidlist[i].vx*asteroidlist[i].vx+asteroidlist[i].vy*asteroidlist[i].vy );
					SetLosingMode();
				}
			}

			// accept input, update position
			if( GetAsyncKeyState(VK_ESCAPE)&0x8000 )
			{ // quit!
				SetMenuMode();
				return;
			}
			if( (GetAsyncKeyState(VK_LEFT)&0x8000) || (GetAsyncKeyState('A')&0x8000))
			{ // turn left
				player_rotation-=0.11;
			}
			if( (GetAsyncKeyState(VK_RIGHT)&0x8000) || (GetAsyncKeyState('D')&0x8000))
			{ // turn right
				player_rotation+=0.11;
			}
			if( (GetAsyncKeyState(VK_UP)&0x8000) || (GetAsyncKeyState('W')&0x8000))
			{ // forwards
				player_vx += cos(player_rotation)*0.5;
				player_vy += sin(player_rotation)*0.5;
			}
			if( (GetAsyncKeyState(VK_DOWN)&0x8000) || (GetAsyncKeyState('S')&0x8000))
			{ // reverse/brake
				player_vx *= 0.7;
				player_vy *= 0.7;
			}
			fire_down = false;
			if( (GetAsyncKeyState(VK_SPACE)&0x8000))
			{ 
				fire_down=true;
				game_autofire_count++;
			}
			if(!fire_down) game_autofire_count=0;
			if( (fire_down && !fire_was_down) || game_autofire_count>=game_autofire_frames )
			{ // shoot! (on transition, make people wear out their spacebars :P)
				ShotFired();
				SpawnBullet( player_x, player_y, player_rotation, game_playersize);
				game_autofire_count=0;
			}
			fire_was_down=fire_down;

			player_x += player_vx*game_playerspeed;
			player_y += player_vy*game_playerspeed;

			player_vx *= 0.9;
			player_vy *= 0.9;

			// collision with outer "wall"

			r = game_playersize * 3.0 / 4.0;
			// check against line segments
			for(i=0;i<screen_boundary.size();i++)
			{
				j=(i+1)%screen_boundary.size();
				dx = screen_boundary[j].x-screen_boundary[i].x;
				dy = screen_boundary[j].y-screen_boundary[i].y;
				d = sqrt(dx*dx+dy*dy);
				if(d==0) continue;
				nx = -dy/d;
				ny = dx/d;
				d = (player_x-screen_boundary[i].x)*nx + (player_y-screen_boundary[i].y)*ny;
				if(d<r && d>=0)
				{ // first check that we're in the line's area of effect
					if( ( (player_x-screen_boundary[i].x)*dx + (player_y-screen_boundary[i].y)*dy) >=0 && 
						( (player_x-screen_boundary[j].x)*dx + (player_y-screen_boundary[j].y)*dy) <=0    )
					{  // move away from line, "bounce" velocity if it was towards line.
						player_x += (r-d)*nx;
						player_y += (r-d)*ny;
						if( (n=(player_vx*nx + player_vy*ny)) <0 )
						{ // was moving towards line, reflect back. 
							player_vx += 2*nx*(-n);
							player_vy += 2*ny*(-n);
						}
					}
				}
			}

			// check against corner points!
			for(i=0;i<screen_boundary.size();i++)
			{
				dx = player_x - screen_boundary[i].x;
				dy = player_y - screen_boundary[i].y;
				d = sqrt(dx*dx+dy*dy);
				if(d<r) 
				{ // move away from corner, "bounce" velocity if it was towards corner.
					player_x = screen_boundary[i].x + dx*r/d;
					player_y = screen_boundary[i].y + dy*r/d;
					if( (n=(player_vx*dx + player_vy*dy)) <0 )
					{ // was moving towards corner, reflect back. 
						player_vx += 2*dx*(-n)/(d*d);
						player_vy += 2*dy*(-n)/(d*d);
					}
				}				
			}

		}

		// render!
		RenderPoint rp[4];
		rp[0].x = game_playersize;
		rp[0].y = 0;
		rp[1].x = -3*game_playersize/4;
		rp[1].y = -game_playersize/2;
		rp[2].x = -game_playersize/4;
		rp[2].y = 0;
		rp[3].x = -3*game_playersize/4;
		rp[3].y = game_playersize/2;;
		playerSprite->bmp.resize((int)(game_playersize+8)*3,(int)(game_playersize+8)*3);
		playerSprite->bmp.cls();
		playerSprite->bx = (int)(floor(player_x)-playerSprite->bmp.wid()/2);
		playerSprite->by = (int)(floor(player_y)-playerSprite->bmp.hei()/2);
		g_Ras->Bind( &playerSprite->bmp, playerSprite->bx - player_x, playerSprite->by - player_y );
		g_Ras->DrawClosedFigure(4,rp,player_rotation);
		g_Ras->FlushChanges();

	}

	if(g_gs==GS_MENU) {


		if( GetAsyncKeyState(VK_ESCAPE)&0x8000 )
		{ // quit!
		//	exit(0); // hee.
		}

		// render menu figure!
		vector<RenderRectangle> rrs;
		rrs.reserve(512);
		RenderRectangle rbd;

		rbd.x = menuwidth/2.0;
		rbd.y = menuheight/2.0;
		rbd.w = menuwidth - 30.0;
		rbd.h = menuheight - 30.0;
		
		g_Ras->Bind( &menuSprite->bmp );

		g_Ras->DrawRectangles(1,&rbd, sin(game_time/3)*0.01 );

		char * text;
		double charwidth;
		double y;
		y = 30;

		text = "DESKTOP ROX";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "A MINIMAL MINIMAL GAME FOR LD11    ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "CONTROLS:                          ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "ARROW KEYS MOVE. SPACE TO SHOOT.   ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "CLICK ANOTHER APP TO PAUSE.        ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "CLICK THE SHIP TO RESUME.          ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;


		text = "ESC TO RETURN TO MAIN MENU         ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		g_Ras->DrawRectangles((int)rrs.size(),&rrs[0], 0);
	
		rrs.clear();
		double h = menuheight - y - 30;
		
		btnstart.x = 30+(menuwidth-60)/4;
		btnstart.y = y+h/2;
		btnstart.w = (menuwidth-60)/2-20;
		btnstart.h = h-20;
		rrs.push_back(btnstart);
		btnexit=btnstart;
		btnexit.x+=(menuwidth-60)/2;
		rrs.push_back(btnexit);

		g_Ras->DrawRectangles((int)rrs.size(),&rrs[0], 0);

		rrs.clear();

		double charheight;
		text = "  START  ";
		charwidth = (btnstart.w-20) / strlen(text);
		charheight = charwidth*5/3.5;
		RenderString(rrs, btnstart.x-btnstart.w/2+10,btnstart.y-charheight/2, charwidth/4.2, charwidth, text);

		text = "  EXIT  ";
		charwidth = (btnexit.w-20) / strlen(text);
		charheight = charwidth*5/3.5;
		RenderString(rrs, btnexit.x-btnexit.w/2+10,btnexit.y-charheight/2, charwidth/4.2, charwidth, text);

		g_Ras->DrawRectangles((int)rrs.size(),&rrs[0], 0);

	} else if(g_gs==GS_LOSING) {


		if( GetAsyncKeyState(VK_ESCAPE)&0x8000 )
		{ // quit!
			SetMenuMode();
		}

		// render menu figure!
		vector<RenderRectangle> rrs;
		rrs.reserve(512);
		RenderRectangle rbd;

		rbd.x = menuwidth/2.0;
		rbd.y = menuheight/2.0;
		rbd.w = menuwidth - 30.0;
		rbd.h = menuheight - 30.0;

		g_Ras->Bind( &menuSprite->bmp );

		g_Ras->DrawRectangles(1,&rbd, sin(game_time/3)*0.01 );

		char * text;
		double charwidth;
		double y;
		y = 30;

		text = " FAILURE ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "IF AT FIRST YOU DONT SUCCEED...    ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = " WELL. GET OUT OF THE WAY NEXT TIME";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "                                   ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "YOU TRIED TO MERGE WITH THE ROCK   ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;

		text = "... NOT A VERY GOOD IDEA           ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5 + 8;


		char stuff[64];
		wsprintf(stuff,"YOUR SCORE WAS: %-9i ", game_score);
		text = stuff;
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5+8;


		text = "CLICK OR PRESS ESC TO CONTINUE.    ";
		charwidth = (menuwidth-60.0) / strlen(text);
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);
		y+=6*charwidth/3.5;


		g_Ras->DrawRectangles((int)rrs.size(),&rrs[0], 0);

	} else {
		// render scoreboard:
		vector<RenderRectangle> rrs;
		rrs.reserve(256);
		RenderRectangle rbd;

		rbd.x = scorewidth/2.0;
		rbd.y = scoreheight/2.0;
		rbd.w = scorewidth - 30.0;
		rbd.h = scoreheight - 30.0;

		g_Ras->Bind( &scoreSprite->bmp );

		g_Ras->DrawRectangles(1,&rbd, sin(game_time/3)*0.01 );


		double charwidth, charheight, y;
		char text[32];
		wsprintf(text,"SCORE:%-9i",game_score);

		charwidth = (scorewidth-60.0) / strlen(text);
		charheight = charwidth*5/3.5;
		RenderString(rrs, 30,40, charwidth/4.2, charwidth, text);


		if(game_difficulty>90)
		{
			wsprintf(text,"DIFFICULTY:OVER 9000!!!");
		}
		else
		{
			wsprintf(text,"DIFFICULTY:%-4i%%      ",(int)(game_difficulty*100));
		}

		charwidth = (scorewidth-60.0) / strlen(text);
		charheight = charwidth*5/3.5;
		y=scoreheight-40-charheight;
		RenderString(rrs, 30,y, charwidth/4.2, charwidth, text);

		g_Ras->DrawRectangles((int)rrs.size(),&rrs[0], 0);

	}


}








int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR cmdline, int nsc)
{
	SetupBlockFont();
	srand(GetTickCount());
	g_hInst = hInst;
	SetSpriteManager(SMT_WINDOW);
	SetRasterizer(RT_ORIGINAL);
	SetupGame();
	SetupMenu();

	playerSprite = AddSprite(ST_BITMAP);
	menuSprite = AddSprite(ST_BITMAP);
	scoreSprite = AddSprite(ST_BITMAP);

	playerSprite->bmp.resize(200,200);
	playerSprite->flags = FLAGS_SETFOCUS;
	menuSprite->bmp.resize(menuwidth,menuheight);
	menuSprite->flags = FLAGS_SETFOCUS|FLAGS_WANTCLICKS;
	menuSprite->bx = menux;
	menuSprite->by = menuy;
	scoreSprite->bmp.resize(scorewidth,scoreheight);
	scoreSprite->flags = FLAGS_SETFOCUS;
	scoreSprite->bx = scorex;
	scoreSprite->by = scorey;


	g_SM->SetClickHandler(ClickHandler);

	SetMenuMode();

	__int64 temp, frame_time_ticks;
	QueryPerformanceFrequency((LARGE_INTEGER *) &temp );
	timer_resolution = 1.0 / temp;
	QueryPerformanceCounter((LARGE_INTEGER *) &start_time );
	last_frame_time = start_time;
	cur_game_time = 0;
	frame_time_ticks = (__int64) ( temp * frame_time );





	MSG msg;
	do {
		if( PeekMessage(&msg,0,0,0,PM_REMOVE) )
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			QueryPerformanceCounter((LARGE_INTEGER *) &temp );
			cur_game_time = temp-start_time;
			game_time = cur_game_time*timer_resolution;
			if( (temp-last_frame_time) >= frame_time_ticks ) {
				last_frame_time=last_frame_time+frame_time_ticks;
				if((temp-last_frame_time)>(frame_time_ticks>>3))
				{
					last_frame_time=temp;
				}
				// run frame!
				UpdateGameLogic();
				g_SM->Redraw();
			}
			else
			{
				Sleep(1);
			}
		}
	} while(msg.message != WM_QUIT);


	return 0;
}